--[[============================================================================
vTableDemo
============================================================================]]--
--[[

User interface for vTableDemo. Extends the vDialog class, which provides
methods for launching the window automatically on startup

]]

class 'vTableDemo_UI' (vDialog)

--------------------------------------------------------------------------------
-- Constructor method

function vTableDemo_UI:__init(...)

  -- when extending a class, it's constructor needs to be called
  -- we also pass the arguments (...) along to the vDialog constructor 
  vDialog.__init(self,...)

  self.vb = renoise.ViewBuilder()
  self.vtable = nil

  self:build()

end

-------------------------------------------------------------------------------
-- Build the UI (executed once)

function vTableDemo_UI:build()
  TRACE("vTableDemo_UI:build()")
  
  local vb = self.vb

  -- we store our complete view in 'vb_content'
  -- (it is used when vDialog.create_dialog() is called)
  self.vb_content = vb:column{
    width = 400,
    margin = 4,
    spacing = 4,
    vb:row{
      vb:checkbox{
        bind = renoise.tool().preferences.autostart
      },
      vb:text{
        text = "Autostart tool when Renoise launches"
      }
    }
  }

  -- call the build_table() method, which will return a vTable object
  -- (the table exposes it's viewbuilder view via the .view property)
  self.vtable = self:build_table()
  self.vtable.show_header = false
  self.vb_content:add_child(self.vtable.view)

end

-------------------------------------------------------------------------------
-- Build the table

function vTableDemo_UI:build_table()

  local img_lock = renoise.tool().bundle_path.."/images/Lock.bmp"

  -- when you click a button, we "look up" the actual data that
  -- is associated with the row. Note that the button is a simple
  -- button, if you need a "toggling" behaviour or custom colors
  -- you need to apply it separately (not part of this simple tutorial)
  local button_click_handler = function(elm,val)
    local item = elm.owner:get_item_by_id(elm.item_id)
    if item then
      renoise.app():show_message("The label for this row is\n "..item.LABEL) 
    end
  end

  -- the column definitions tell the table something about each column
  -- you can also pass properties (e.g. font,color) and the cells will  
  -- use that (assuming of course that it understands the property...) 
  local col_defs = {
    {key = "LABEL", col_width="auto", col_type=vTable.CELLTYPE.TEXT, align="right", tooltip="This is text/label"},
    {key = "LABEL2", col_width=90, col_type=vTable.CELLTYPE.TEXT, font="bold", align="right", tooltip="This is text/label"},
    {key = "BUTTON", col_width=30, col_type=vTable.CELLTYPE.BUTTON, bitmap=img_lock, color={0x00,0x60,0x00}, notifier=button_click_handler, tooltip="This is a button"},      
  }

  -- now return an instance of vTable   
  return vTable{
    id = "vTable",
    vb = self.vb,
    column_defs = col_defs,
    cell_style = "invisible", -- see viewbuilder API > rack.style
    scrollbar_width = 18,
    row_height = 18,
    num_rows = 10,
    width = 550,    
    --height = 300, -- automatically determined by number of rows
  }

end

-------------------------------------------------------------------------------
-- just some example data...
-- each value is indicated by the "key" in vtable.column_defs 

function vTableDemo_UI:set_vtable_data()

  local data = {
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Doofer  ➡  Gainer  ➡  Low EQ 10  ➡  Medium EQ 10  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Doofer  ➡  Low EQ 10  ➡  Medium EQ 10  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Doofer  ➡  Medium EQ 10  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Doofer  ➡  Gainer  ➡  Medium EQ 10  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Doofer  ➡  Gainer  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Doofer  ➡  Medium EQ 10  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Gainer  ➡  Low EQ 10  ➡  Medium EQ 10  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Low EQ 10  ➡  Medium EQ 10  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Medium EQ 10  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Doofer  ➡  High EQ 10 "},
    {BUTTON = "", LABEL2 = "EQ 30 + Gainer", LABEL = "Doofer  ➡  Gainer  ➡  Medium EQ 10  ➡  High EQ 10 "},
  }

  -- setting the data will update the table
  self.vtable.data = data

end

-------------------------------------------------------------------------------
-- methods required by vDialog
-------------------------------------------------------------------------------
-- return the UI which was previously created with build()

function vTableDemo_UI:create_dialog()
  TRACE("vTableDemo_UI:create_dialog()")

  return self.vb_content

end

